/*
 *
 *
 *
 *
 * Program Memory is paged only in the upper half (>=0x2000).  The lower
 * half is constant.
 *
 *
 */

#include "sim.h"

/*
 * pmsetbyte(addr, val)
 *
 * We wrap this around all the array writes so that we both handle
 * paging in our own way and so that we can watch for and trap memory
 * accesses for debugging.
 *
 * Program Memory is paged in the upper half only.  The lower half (<0x2000)
 * stays constant.
 *
 */
void pmsetbyte(unsigned int addr, unsigned char val)
{
  if (addr < 0x2000)
    {
      curdsp.pm[addr] = val;
    }
  else
    {
      fprintf(stderr, "pmsetbyte: unsupported paged write\n");
    }
  /* TODO: add some stuff here for trapping */

  return;
}

unsigned char pmgetbyte(unsigned int addr)
{
  if (addr < (0x2000*3))
    {
      return curdsp.pm[addr];
    }
  else
    {
      fprintf(stderr, "pmgetbyte: unsupported paged read\n");
    }
  /* TODO: add stuff for trapping */
  return 0;
}

unsigned int pmgetiword(unsigned int waddr)
{
  unsigned int baddr;

  baddr = waddr*3;
  
  if (baddr < (0x2000*3))
    {
      return glue24(curdsp.pm[baddr],curdsp.pm[baddr+1],curdsp.pm[baddr+2]);
    }
   else
    {
      fprintf(stderr, "pmgetiword: unsupported paged read\n");
    }
  /* TODO: add stuff for trapping */
  return 0;
}

void pmsetiword(unsigned int waddr, unsigned int val)
{
  unsigned int baddr;
  baddr = waddr*3;
  if (baddr < (0x2000*3))
    {
      curdsp.pm[baddr++] = (val >> 16) & 0xff;
      curdsp.pm[baddr++] = (val >> 8) & 0xff;
      curdsp.pm[baddr] = val & 0xff;
    }
  else
    {
      fprintf(stderr, "pmgetiword: unsupported paged read\n");
    }
  /* TODO: add stuff for trapping */
  return;
}

#define MAXDECODE 100

/*
 * start, len are in 24bit words
 */
int pmdump(int start, int len)
{
  int i;
  char buf[MAXDECODE+1];
  unsigned int op = 0xffffff;
  
  fprintf(stderr, " --- Program Memory Dump at %04x to %04x --- \n", start, start+len);
  if ((start+len) >= 0x2000)
    {
      fprintf(stderr, "pmdump: unsupported paged dump\n");
      return -1;
    }
  else
    {
      for (i=0; (i < len) && ((start+len)<(DM_SIZE/3));i++)
	{
	  op = pmgetiword(start+i);
	  fprintf(stderr, "0x%04x: %06x\t", start+i, op);
	  if (prettydecode(op, buf) < 0)
	    fprintf(stderr, "error\n");
	  else
	    fprintf(stderr, "%s\n", buf);
	}
    }
  fprintf(stderr, " ------------------------------------------- \n");

  return 0;
}

