/*
 * This contains roughly the same structure as the main decoder,
 * except that instead of executing the instruction, we write 
 * an assembler-like representation of it to a passed buffer.
 *
 */

#include "sim.h"
#include "pretty_tables.h"

typedef unsigned int (*opprettydecode_f)(unsigned int, char *);

opprettydecode_f prettydecodetable[] = {
  NULL,
  prettyop_pmdmr,
  prettyop_dmwim,
  prettyop_dmrwim,
  prettyop_dmrw,
  prettyop_pmrw,
  prettyop_loadregim,
  prettyop_loadnondataregim,
  prettyop_alumove,
  prettyop_alu,
  prettyop_jump,
  prettyop_dountil,
  prettyop_shiftdmrw,
  prettyop_shiftpmrw,
  prettyop_shiftmove,
  prettyop_shiftim,
  prettyop_shift,
  prettyop_datamove,
  prettyop_modectl,
  prettyop_jumpindirect,
  prettyop_return,
  prettyop_modify,
  prettyop_trap,
  prettyop_divq,
  prettyop_divs,
  prettyop_satmr,
  prettyop_stackctl,
  prettyop_jumpfi,
  prettyop_modifyfl,
  prettyop_iorw,
  prettyop_nop,
  prettyop_idle,
  prettyop_idlen
};



/*
 * Returns the reg/rgp for the register name (of len namelen).
 *
 */
int findreg(char *name, int namelen, int *reg, int *rgp)
{
  int i,j;
  if (!name || (!reg || !rgp))
    return 0;

  for (i=0;i<16;i++)
    {
      for (j=0;j<4;j++)
	{
	  if (!strncmp(name, regs[i][j], namelen))
	    {
	      *reg = i;
	      *rgp = j;
	      return 1;
	    }
	}
    }
  
  return 0;
}

unsigned short getconstant(unsigned int yy, unsigned int cc, unsigned int bo)
{
  if(bo==1)
    {
      switch(yy)
	{
	case 0x00:
	  switch(cc)
	    {
	    case 0x00: return 0x0001;
	    case 0x01: return 0x0002;
	    case 0x02: return 0x0004;
	    case 0x03: return 0x0008;
	    }
	case 0x01:
	  switch(cc)
	    {
	    case 0x00: return 0x0010;
	    case 0x01: return 0x0020;
	    case 0x02: return 0x0040;
	    case 0x03: return 0x0080;
	    }
	case 0x02:
	  switch(cc)
	    {
	    case 0x00: return 0x0100;
	    case 0x01: return 0x0200;
	    case 0x02: return 0x0400;
	    case 0x03: return 0x0800;
	    }
	case 0x03:
	  switch(cc)
	    {
	    case 0x00: return 0x1000;
	    case 0x01: return 0x2000;
	    case 0x02: return 0x4000;
	    case 0x03: return 0x8000;
	    }
	}
    }
  else if (bo==3)
    {
      switch(yy)
	{
	case 0x00:
	  switch(cc)
	    {
	    case 0x00: return 0xfffe;
	    case 0x01: return 0xfffd;
	    case 0x02: return 0xfffb;
	    case 0x03: return 0xfff7;
	    }
	case 0x01:
	  switch(cc)
	    {
	    case 0x00: return 0xffef;
	    case 0x01: return 0xffdf;
	    case 0x02: return 0xffbf;
	    case 0x03: return 0xff7f;
	    }
	case 0x02:
	  switch(cc)
	    {
	    case 0x00: return 0xfeff;
	    case 0x01: return 0xfdff;
	    case 0x02: return 0xfbff;
	    case 0x03: return 0xf7ff;
	    }
	case 0x03:
	  switch(cc)
	    {
	    case 0x00: return 0xefff;
	    case 0x01: return 0xdfff;
	    case 0x02: return 0xbfff;
	    case 0x03: return 0x7fff;
	    }
	}
    }
  else
    fprintf(stderr, "pretty: unknown bo\n");
  return 0;
}


/*
 * 
 *
 */
int prettydecode(unsigned int opcode, char *buf)
{
  int type = 0;

  if (!buf)
    return -1;
  
  memset(buf, 0x00, MAXDECODE+1);

  if (opcode & BIT23) /* 1 */
    {
      if (opcode & BIT22) /* 11 -- type 1 */
	type = 11;
      else /* 10 */
	{
	  if (opcode & BIT21) /* 101 -- type 2 */
	    type = 2;
	  else /* 100 -- type 3 */
	    type = 3;
	}
    }
  else /* 0 */
    {
      if (opcode & BIT22) /* 01 */
	{
	  if (opcode & BIT21) /* 011 -- type 4 */
	    type = 4;
	  else /* 010 */
	    {
	      if (opcode & BIT20) /* 0101  -- type 5 */
		type = 5;
	      else /* 0100 -- type 6 */
		type = 6;
	    }
	}
      else  /* 00 */
	{
	  if (opcode & BIT21) /* 001 */
	    {
	      if (opcode & BIT20) /* 0011 -- type 7 */
		type = 7;
	      else /* 0010 */
		{
		  if (opcode & BIT19) /* 0010 1 -- type 8 */
		    type = 8;
		  else /* 0010 0 -- type 9 */
		    type = 9;
		}
	    }
	  else /* 000 */
	    {
	      if (opcode & BIT20) /* 0001 */
		{
		  if (opcode & BIT19) /* 00011 -- type 10 */
		    type = 10;
		  else /* 00010 */
		    {
		      if (opcode & BIT18) /* 000101 -- type 11 */
			type = 11;
		      else /* 000100 */
			{
			  if (opcode & BIT17) 
			    type = 12; /* type 12 */
			  else
			    {
			      if (opcode & BIT16)
				type = 13; /* type 13 */
			      else
				type = 14; /* type 14 */
			    }
			}
		    }
		}
	      else /* 0000 */
		{
		  if (opcode & BIT19) /* 0000 1 */
		    {
		      if (opcode & BIT18) /* 0000 11 */
			{
			  if (opcode & BIT17) /* 0000 111 */
			    {
			      if (opcode & BIT16) /* 0000 1111 */
				{
				  if (opcode & BIT15) 
				    ;
				  else /* type 15 */
				    type = 15;
				}
			      else /* 0000 1110 */
				{
				  if (opcode & BIT15) /* 0000 1110 1 */
				    ;
				  else /* type 16 */
				    type = 16;
				}
			    }
			  else /* 0000 110 */
			    {
			      if (opcode & BIT16) /* 0000 1101 -- type 17 */
				type = 17;
			      else /* 0000 1100 */
				type = 18; /* type 18 */
			    }
			}
		      else /* 0000 10 */
			{
			  if ((opcode >> 4) == 0xa001) /* rti */
			    type = 20; /* type 20 */
			  else if ((opcode >> 4) == 0xa000) /* rts */
			    type = 20; /* type 20 */
			  else if ((opcode >> 4) == 0x09080)
			    type = 21; /* type 21 */
			}
		    }
		  else /* 0000 0 */
		    {
		      if (opcode & BIT18) /* 0000 01 */
			{
			  if ((opcode >> 5) == 0x2000)
			    type = 26; /* type 26 */
			  else if (opcode & BIT17) /* 0000 011 */
			    {
			      if (opcode & BIT16) /* 0000 0111 */
				type = 23; /* type 23 */
			      else /* 0000 0110 */
				type = 24; /* type 24 */
			    }
			  else if (opcode == 0x050000)
			    type = 25; /* type 25 */
			}
		      else /* 000000 */
			{
			  if (opcode & BIT17) /* 0000 001 */
			    {
			      if (opcode & BIT16) /* 0000 0011 -- type 27 */
				type = 27;
			      else /* 0000 0010 -- type 28 */
				type = 28;
			    }
			  else /* 0000 000 */
			    {
			      if (opcode & BIT16) /* 0000 0001 -- type 29 */
				type = 29;
			      else /* 0000 0000 */
				{
				  if (opcode == 0x000000)
				    type = 30; /* type 30 */
				  else if ((opcode>>4) == 0x02800) 
				    {
				      if (opcode & 0xf)
					type = 32; /* type 31 */
				      else
					type = 31; /* type 31 */
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }

  if (type && (type <= 32))
    return (*(prettydecodetable[type]))(opcode,buf);
  
  sprintf(buf, "pretty: unknown instruction");
  return -1;
}
